/*       Version de base du programme Schrodinger à une dimension
Utilise un programme de diagonalisation de la bibliothèque LAPACK
         Compiler avec : c++ schro_simple.cpp -llapack -o schro_simple
*/

#include <iostream>
#include <fstream>
#include <string>
#include <cmath>

using namespace std ;

/* Lapack est en fortran, or pour l'appel d'un programme fortran, il faut
plusieurs choses :
 -il faut ajouter '_' (underscore) à la fin de son nom (donc 'ssteqr' devient
'ssteqr_'
 -il faut transmettre les arguments scalaires comme des pointeurs (pas les
tableaux qui sont déjà des pointeurs)
 -il faut faire un prototype en mentionnant extern "C"  */

extern "C"
void ssteqr_( char * , int *, float [], float [], float [], int *, float [],
              int *  ) ;

void  tridiag( int n, float d[], float e[], float z[] )
/* Appel du sous-progpramme ssteqr de Lapack : calcul des valeurs et vecteurs
propres d'une matrice tridiagonale symmetrique   */
{
    int info ; // diagnostic
    float work[2*n-2] ; // espace de travail
    char compz='v' ; // mettre 'n' pour n'avoir que les valeurs propres

    ssteqr_( &compz, &n, d, e, z, &n, work, &info ) ; // scalaires pointeurs
    if ( info == 0 ) cout << "La diagonalisation s'est bien passée" << endl ;
    else cout << "Attention ! SSTEQR : info =" << info << endl ;
}

int main(void) /* programme principal */
{
    int const n = 500 ; // nombre de points de discretisation
    /* astuce "à la" C++ : au lieu de déclarer un tableau bidimensionnel z[n][n]
    comme on le ferait tout naturellement en fortran, il est plus simple de
    déclarer un tableau unidimensionnel z[n*n] : ça évite d'avoir à déclarer n
    en constante globale avant la fonction tridiag, et ça évite aussi d'avoir à se
    battre avec l'ordre des indices du C opposé à celui du Fortran*/
    float d[n], e[n-1], z[n*n], x[n], v[n] ;

    for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
            if ( i == j )  z[i+n*j] = 1. ;
            else z[i+n*j] = 0. ;  // matrice identite
        }
    }

    float dx = 20./n ; // l'échantillon est l'intervalle [-10,10]

    for( int i = 0; i < n; i++ ) {
        x[i] = (i-n/2)*dx ;
        v[i] = x[i]*x[i] ; // definition du potentiel : ici harmonique
        if ( i < n-1 ) e[i] = -1./(dx*dx) ;
        d[i] = 2./(dx*dx)+v[i] ;
    }

    tridiag (n, d, e, z);  // calcul des valeurs et vecteurs propres

    return 0;
}
